@@ -1,4 +1,5 @@
-use crate::value::{FromArma, IntoArma, Value};
+use crate::ext_result::IntoExtResult;
+use crate::value::{FromArma, Value};
 use crate::Context;
 
 type HandlerFunc = Box<
@@ -11,11 +12,14 @@ type HandlerFunc = Box<
     ) -> libc::c_int,
 >;
 
-pub(crate) struct Handler {
-    pub(crate) handler: HandlerFunc,
+/// A wrapper for `HandlerFunc`
+pub struct Handler {
+    /// The function to call
+    pub handler: HandlerFunc,
 }
 
-pub(crate) fn fn_handler<C, I, R>(command: C) -> Handler
+/// Create a new handler from a Factory
+pub fn fn_handler<C, I, R>(command: C) -> Handler
 where
     C: Factory<I, R> + 'static,
 {
@@ -33,7 +37,8 @@ where
     }
 }
 
-pub(crate) trait Executor: 'static {
+/// Execute a command
+pub trait Executor: 'static {
     /// # Safety
     /// This function is unsafe because it interacts with the C API.
     unsafe fn call(
@@ -49,7 +54,7 @@ pub(crate) trait Executor: 'static {
 /// A factory for creating a command handler.
 /// Creates a handler from any function that optionally takes a context and up to 12 arguments.
 /// The arguments must implement `FromArma`
-/// The return value must implement `IntoArma`
+/// The return value must implement `IntoExtResult`
 pub trait Factory<A, R> {
     /// # Safety
     /// This function is unsafe because it interacts with the C API.
@@ -183,7 +188,7 @@ macro_rules! factory_tuple ({ $c: expr, $($param:ident)* } => {
     // No context with input and return
     impl<Func, $($param,)* R> Factory<($($param,)*), R> for Func
     where
-        R: IntoArma + 'static,
+        R: IntoExtResult + 'static,
         Func: Fn($($param),*) -> R,
         $($param: FromArma,)*
     {
@@ -195,22 +200,11 @@ macro_rules! factory_tuple ({ $c: expr, $($param:ident)* } => {
                 return format!("2{}", count).parse::<libc::c_int>().unwrap();
             }
             if $c == 0 {
-                if crate::write_cstr(
-                    {
-                        let ret = (self)($($param::from_arma("".to_string()).unwrap(),)*);
-                        if let Value::String(s) = ret.to_arma() {
-                            s
-                        } else {
-                            ret.to_arma().to_string()
-                        }
-                    },
+                handle_output_and_return(
+                    (self)($($param::from_arma("".to_string()).unwrap(),)*),
                     output,
                     size
-                ).is_none() {
-                    4
-                } else {
-                    0
-                }
+                )
             } else {
                 #[allow(unused_variables, unused_mut)]
                 let mut argv: Vec<String> = {
@@ -228,32 +222,23 @@ macro_rules! factory_tuple ({ $c: expr, $($param:ident)* } => {
                     argv.reverse();
                     argv
                 };
-                if crate::write_cstr(
+                #[allow(unused_variables, unused_mut)] // Caused by the 0 loop
+                let mut c = 0;
+                #[allow(unused_assignments, clippy::eval_order_dependence)]
+                handle_output_and_return(
                     {
-                        #[allow(unused_variables, unused_mut)] // Caused by the 0 loop
-                        let mut c = 0;
-                        #[allow(unused_assignments, clippy::eval_order_dependence)]
-                        let ret = (self)($(
+                        (self)($(
                             if let Ok(val) = $param::from_arma(argv.pop().unwrap()) {
                                 c += 1;
                                 val
                             } else {
                                 return format!("3{}", c).parse::<libc::c_int>().unwrap()
                             },
-                        )*);
-                        if let Value::String(s) = ret.to_arma() {
-                            s
-                        } else {
-                            ret.to_arma().to_string()
-                        }
+                        )*)
                     },
                     output,
                     size
-                ).is_none() {
-                    4
-                } else {
-                    0
-                }
+                )
             }
         }
     }
@@ -261,7 +246,7 @@ macro_rules! factory_tuple ({ $c: expr, $($param:ident)* } => {
     // Context with input and return
     impl<Func, $($param,)* R> Factory<(Context, $($param,)*), R> for Func
     where
-        R: IntoArma + 'static,
+        R: IntoExtResult + 'static,
         Func: Fn(Context, $($param),*) -> R,
         $($param: FromArma,)*
     {
@@ -273,22 +258,11 @@ macro_rules! factory_tuple ({ $c: expr, $($param:ident)* } => {
                 return format!("2{}", count).parse::<libc::c_int>().unwrap();
             }
             if $c == 0 {
-                if crate::write_cstr(
-                    {
-                        let ret = (self)(context, $($param::from_arma("".to_string()).unwrap(),)*);
-                        if let Value::String(s) = ret.to_arma() {
-                            s
-                        } else {
-                            ret.to_arma().to_string()
-                        }
-                    },
+                handle_output_and_return(
+                    (self)(context, $($param::from_arma("".to_string()).unwrap(),)*),
                     output,
                     size
-                ).is_none() {
-                    4
-                } else {
-                    0
-                }
+                )
             } else {
                 #[allow(unused_variables, unused_mut)]
                 let mut argv: Vec<String> = {
@@ -306,37 +280,61 @@ macro_rules! factory_tuple ({ $c: expr, $($param:ident)* } => {
                     argv.reverse();
                     argv
                 };
-                if crate::write_cstr(
+                #[allow(unused_variables, unused_mut)] // Caused by the 0 loop
+                let mut c = 0;
+                #[allow(unused_assignments, clippy::eval_order_dependence)]
+                handle_output_and_return(
                     {
-                        #[allow(unused_variables, unused_mut)] // Caused by the 0 loop
-                        let mut c = 0;
-                        #[allow(unused_assignments, clippy::eval_order_dependence)]
-                        let ret = (self)(context, $(
+                        (self)(context, $(
                             if let Ok(val) = $param::from_arma(argv.pop().unwrap()) {
                                 c += 1;
                                 val
                             } else {
                                 return format!("3{}", c).parse::<libc::c_int>().unwrap()
                             },
-                        )*);
-                        if let Value::String(s) = ret.to_arma() {
-                            s
-                        } else {
-                            ret.to_arma().to_string()
-                        }
+                        )*)
                     },
                     output,
                     size
-                ).is_none() {
-                    4
-                } else {
-                    0
-                }
+                )
             }
         }
     }
 });
 
+unsafe fn handle_output_and_return<R>(
+    ret: R,
+    output: *mut libc::c_char,
+    size: libc::size_t,
+) -> libc::c_int
+where
+    R: IntoExtResult + 'static,
+{
+    let ret = ret.to_ext_result();
+    let ok = ret.is_ok();
+    if crate::write_cstr(
+        {
+            let value = match ret {
+                Ok(x) | Err(x) => x,
+            };
+            match value {
+                Value::String(s) => s,
+                v => v.to_string(),
+            }
+        },
+        output,
+        size,
+    )
+    .is_none()
+    {
+        4
+    } else if ok {
+        0
+    } else {
+        9
+    }
+}
+
 factory_tuple! { 0, }
 factory_tuple! { 1, A }
 factory_tuple! { 2, A B }
