@@ -104,6 +104,26 @@ fn sub_group_command_with_args() {
     assert_eq!(result, "Hello John");
 }
 
+#[test]
+fn result_ok() {
+    let extension = Extension::build()
+        .command("hello", || -> Result<&str, &str> { Ok("Hello") })
+        .finish()
+        .testing();
+    let (result, _) = unsafe { extension.call("hello", None) };
+    assert_eq!(result, "Hello");
+}
+
+#[test]
+fn result_err() {
+    let extension = Extension::build()
+        .command("hello", || -> Result<&str, &str> { Err("Error") })
+        .finish()
+        .testing();
+    let (result, _) = unsafe { extension.call("hello", None) };
+    assert_eq!(result, "Error");
+}
+
 #[test]
 fn not_found() {
     let extension = Extension::build().finish().testing();
@@ -169,3 +189,23 @@ fn output_overflow_with_args() {
     let (_, code) = unsafe { extension.call("hello", Some(vec![String::from('X')])) };
     assert_eq!(code, 4);
 }
+
+#[test]
+fn application_error_ok() {
+    let extension = Extension::build()
+        .command("hello", || -> Result<&str, &str> { Ok("Ok") })
+        .finish()
+        .testing();
+    let (_, code) = unsafe { extension.call("hello", None) };
+    assert_eq!(code, 0);
+}
+
+#[test]
+fn application_error_err() {
+    let extension = Extension::build()
+        .command("hello", || -> Result<&str, &str> { Err("Error") })
+        .finish()
+        .testing();
+    let (_, code) = unsafe { extension.call("hello", None) };
+    assert_eq!(code, 9);
+}
